@@ -23,8 +23,10 @@ from _config import *
 
 from _filterbots import *
 from _geoip import *
+from _join import *
 from _merge import *
 from _parse import *
+from _urlparse import *
 from _plot import *
 from _qps import *
 from _sample import *
