@@ -23,7 +23,7 @@ from StringIO import StringIO
 from operator import itemgetter
 
 from logtools import (filterbots, geoip, logsample, logsample_weighted, 
-                      logparse, logmerge, logplot, qps)
+                      logparse, urlparse, logmerge, logplot, qps)
 from logtools.parsers import *
 from logtools import logtools_config, interpolate_config, AttrDict
 
@@ -35,6 +35,22 @@ class ConfigurationTestCase(unittest.TestCase):
         self.assertRaises(KeyError, interpolate_config, None, 'bogus_sec', 'bogus_key')
 
 
+class URLParseTestCase(unittest.TestCase):
+    def setUp(self):
+        self.rows = [
+            "http://www.mydomain.com/my/path/myfile?myparam1=myval1&myparam2=myval2",
+            "http://www.mydomain2.com",
+            "http://www.mydomain3.com/home",
+            "http://fun.com/index.php?home"
+        ]
+        
+    def testUrlParse(self):
+        i=0
+        for row in urlparse(StringIO('\n'.join(self.rows)+'\n'), part='netloc'):
+            i+=1
+        self.assertEquals(i, len(self.rows), \
+                          "Number of rows output is not equal to input size")
+    
 class ParsingTestCase(unittest.TestCase):
     def setUp(self):
         self.clf_rows = [
@@ -67,7 +83,7 @@ class ParsingTestCase(unittest.TestCase):
             self.assertNotEquals(parsed, None, "Could not parse line: %s" % str(logrow))        
         
     def testLogParse(self):
-        options = AttrDict({'parser': 'CommonLogFormat', 'field': 4})
+        options = AttrDict({'parser': 'CommonLogFormat', 'field': 4, 'header': False})
         fh = StringIO('\n'.join(self.clf_rows))
         output = [l for l in logparse(options, None, fh)]
         self.assertEquals(len(output), len(self.clf_rows), "Output size was not equal to input size!")
@@ -84,9 +100,12 @@ class FilterBotsTestCase(unittest.TestCase):
     def setUp(self):
         self.options = AttrDict({
             "reverse": False,
+            "unescape": False,
             "printlines": False,
             "ip_ua_re": "^(?P<ip>.*?) - USER_AGENT:'(?P<ua>.*?)'",
-            "bots_ips": StringIO(),
+            "bots_ips": StringIO("\n".join([
+                "6.6.6.6"
+            ]) + "\n"),
             "bots_ua": StringIO("\n".join([
                 "## Example comment ##",
                 "Mozilla/5.0 (compatible; Googlebot/2.1; +http://www.google.com/bot.html)",
@@ -107,8 +126,24 @@ class FilterBotsTestCase(unittest.TestCase):
             "5.5.5.5 - USER_AGENT:'Java/1.6.0_18'\n" \
             "6.6.6.6 - USER_AGENT:'ssearch_bot/Nutch-1.0 (sSearch Crawler; http://www.semantissimo.de)'\n"
         )
+        self.json_fh = StringIO(
+            '''{"timestamp":"2010\/09\/01 00:00:01","user_agent":"Mozilla\/5.0 (compatible; Googlebot\/2.1; +http:\/\/www.google.com\/bot.html)","user_ip":"66.249.71.108"}\n''' \
+            '''{"timestamp":"2010\/10\/01 11:00:01","user_agent":"Mozilla\/5.0 (compatible; Googlebot\/2.1; +http:\/\/www.google.com\/bot.html)","user_ip":"66.249.71.109"}\n''' \
+            '''{"timestamp":"2010\/09\/01 00:00:01","user_agent":"Mozilla\/5.0 (Windows; U; Windows NT 6.0; en-US; rv:1.9.1.11) Gecko\/20100701 Firefox\/3.5.11 (.NET CLR 3.5.30729)","user_ip":"100.100.1.100"}\n''' \
+            '''{"timestamp":"2010\/10\/01 00:00:01","user_agent":"Mozilla\/5.0 (Windows; U; Windows NT 6.0; en-US; rv:1.9.1.11) Gecko\/20100701 Firefox\/3.5.11 (.NET CLR 3.5.30729)","user_ip":"6.6.6.6"}\n''' \
+        )
 
-    def testFiltering(self):
+    def testParserFiltering(self):
+        json_options = self.options
+        json_options['parser'] = 'JSONParser'
+        json_options['ip_ua_fields'] = 'ua:user_agent,ip:user_ip'
+        
+        i=0
+        for l in filterbots(fh=self.json_fh, **json_options):
+            i+=1
+        self.assertEquals(i, 1, "filterbots output size different than expected: %s" % str(i))
+            
+    def testRegExpFiltering(self):
         i=0
         for l in filterbots(fh=self.fh, **self.options): 
             i+=1
